# 云城平台能力

云城app现处于不断的开发迭代中，目前已提供了如下的相关设备能力的调用方法，如无法满足三方服务的功能需求，请及时与云城app对接人沟通，以便获得云城app的支持，确保功能完整。

# 1.weex子服务可调用能力

云城weex子服务的开发中，开发者可以通过 `weex.requireModule('moduleName')` 引入云城提供的插件，方便快捷的调起云城能力，如监听安卓机物理返回键点击，调用云城扫描二维码等。

## 1.1.监听安卓物理返回键点击事件

``` javascript
// vue
<script>
const globalEvent = weex.requireModule('globalEvent')

export default {
  created() {
    globalEvent.addEventListener("clickEvent", (e) => {
      // do something
    })
  }
}
</script>
```

## 1.2.扫描或生成二维码

``` javascript
// vue
<script>
const wxQRCode = weex.requireModule('wxQRCode')

export default {
  methods: {
    // 扫描二维码
    scanQRCode() {
      wxQRCode.scanQRCode(res => {
        // 返回结果： { res： "1", result: "二维码内容" }
      })
    },
    // 生成二维码
    createQRCode() {
      const param={"width":"100","height":"100","content": "二维码内容"}
      wxQRCode.createQRCode(param, res => {
        // 返回结果： { res： "1", path: "二维码路径" }
      })
    }
  }
}
</script>
```

## 1.3.上传图片

``` javascript
// vue
<script>
  const uploadFile = weex.requireModule("fileUpload")

  export default {
    methods: {
      uploadFile() {
        const params = {
          "URL": "后台服务地址",
          "file": "本地图片地址"
        }
        uploadFile.uploadFile(params, ret => {
          // 返回结果： { res: '1', returnJson: '后台返回信息' }
        })
      }
    }
  }

</script>
```

## 1.4.从相册选择图片

``` javascript
// vue
<script>
  const wxImage = weex.requireModule("image")

  export default {
    methods: {
      pickImage() {
        const type = 1 // 1:图片 2:视频 3:图片和视频
        const count = 9 // 选择的数量限制，最大为9
        wxImage.openImageVideoPicker(type, count, obj => {
          // 返回结果 { res: '1', photos: ['图片路径数组'] }
        })
      }
    }
  }
</script>
```

## 1.5.拍照

``` javascript
// vue
<script>
  const wxCamera = weex.requireModule("camera")

  export default {
    methods: {
      takePhoto() {
        wxCamera.takePicture(obj => {
          // 返回结果 {'res' : '1'， 'photos' : ['图片路径数组']}
        })
      }
    }
  }
</script>
```
<!-- takeVideo() {
  // duration为拍摄最长时间，目前支持的最大值为30，最小值为5
  // res,0为用户取消，1为用户拍摄图片，2为用户拍摄小视频
  // imagePath,当res为1，代表图片地址;当res为2,代表视频缩略图
  // videoPath,视频地址
  const duration = 30
  wxCamera.takeCamera(duration, obj => {
    // 返回结果 {'res' : '2'， 'imagePath' : 缩略图地址, 'videoPath' : 视频地址}
  })
} -->

## 1.6.预览图片

``` javascript
// vue
<script>
  const image = weex.requireModule("image")

  export default {
    methods: {
      preview() {
        // 从数组的第一张开始预览
        const images =['图片路径']
        image.preview(images)
      },
      previewForIndex() {
        // 按指定的数组下标开始预览
        const images =['图片路径']
        image.previewForIndex(images, index)
      }
    }
  }
</script>
```

<!-- 更多云城可调用能力，详见[移动平台插件文档](http://xtwiki.vanke.com/pages/viewpage.action?pageId=4559350)。 -->

## 注意

weex服务中调用了云城能力后如需进行真机调试可有如下方式：
1. 确保手机和开发使用中的电脑在同一网段，使用云城APP扫一扫功能，扫描开发启用服务下的web页面中的二维码。
  ![scanQRCode](/images/scanQRCode.png)

  web页面中的二维码

  ![page](/images/weexPage.png)

  通过云城打开服务页面后即可开始真机调试。

2. 上传打包后文件到[开放平台（生产环境）](https://icloudcity.com/iot-open)或[开放平台（uat环境）](http://icloudcitytest.vanke.com/iot-open/#/home)

# 2.H5子服务可调用能力

在H5的子服务的开发过程中，云城app也提供了一些可以方便使用的设备能力供子服务开发者调用。

## 2.1.调用三方支付

云城app中支持h5调用三方支付，完成h5支付功能。现已支持支付宝及微信支付，具体支付处理逻辑由h5子服务自身实现。<br />
**说明：**

1. 支付宝h5支付请按照[支付宝官方开发文档](https://docs.open.alipay.com/203)进行接入即可。

2. 微信h5支付请按照[微信官方开发文档](https://pay.weixin.qq.com/wiki/doc/api/H5.php?chapter=15_1)接入，并在h5子服务版本发布的入口地址填写时拼接上 `referer="h5子服务于微信开放平台商户申请H5时提交的授权域名"`，云城app已按照[微信官方开发文档](https://pay.weixin.qq.com/wiki/doc/api/H5.php?chapter=15_4)介绍于webview中调用微信h5支付时，将会自动配置Referer请求头。微信h5支付后，iOS回调自动打开浏览器加载回调页面未跳回云城app，详细解决办法可参考[F.A.Q](/pages/development/support.md#8ios端h5子服务中调用微信h5支付后如何回跳至云城app中？)。

![referer](/images/referer.png)

## 2.2.获取地理位置信息

云城app实现了获取用户当前位置经纬度的js调用方法，此方法需用户打开云城app获取位置的权限。
调用方法后，app优先获取GPS定位信息，若失败获取网络位置信息，若都失败则返回获取定位失败的信息。
定位结果以json字符串的形式返回，需要子服务开发人员自行处理。

### 2.2.1.安卓端获取用户定位信息

安卓端提供的js方法会挂载到window.ycapp对象上，子服务开发人员请确保window对象上的ycapp属性未被使用，且并不会覆盖重写此属性。调用方法如下：

```javascript
let location = JSON.parse(window.ycapp.getSystemParam());

// 定位成功返回
{
  "latitude":"22.525918", // 纬度
  "longitude":"113.940788", // 精度
  "res":1
}

// 定位失败返回
{
  "res":0
}
```

### 2.2.2.iOS端获取用户定位信息

iOS端提供的js方法会挂载到window.getLocation属性上，子服务开发人员请确保window对象上的getLocation属性未被使用，且并不会覆盖重写此属性。调用方法如下：

```javascript
window.getLocation = function(location) {
  // 此处为获取到位置信息的回调
  location = JSON.parse(location);
  // do something
}

// 通知app传递位置信息给子服务
window.webkit.messageHandlers.getLocation.postMessage(null);

// 定位成功返回
{
  "latitude":"22.525918", // 纬度
  "longitude":"113.940788", // 精度
  "msg":"成功"
  "res":1
}

// 定位失败返回
{
  "res":0,
  "msg":"失败"
}
```



## 2.3.拨打电话

云城app实现了标准的h5拨打电话功能，在h5页面中想要调用手机的拨号功能，只需按照h5标准书写即可。如：

```html
<a href="tel:10086">致电中国移动</a>
```

## 2.4.h5调用云城支付

![pay](/images/pay.jpg)

<img src="/images/effect.png" width="450" />

云城app拥有自己的支付通道，因此也为子服务提供对应的调用支付的能力，h5子服务可在页面内调用对应的方法唤起app支付功能。现已支持微信支付，更多支付功能正在接入中，具体实现代码如下：

参数名称|类型|是否必须|描述
:--|:--|:--|:--
merchantNo|string|true|云城app分配的商户号，需与云城app对接
tradeOrderNo|string|true|业务订单号，1-32位，只能为英文、数字、下划线
amount|number|true|订单金额（0.01-20000），最多2位小数
notifyUrl|string|true|支付回调的地址，支付完成后，会回调这个地址(选填)
expireTimeMinute|number|true|订单过期时间，单位分钟，不低于5分钟，不高于1440分钟
goodsDesc|string|true|商品描述，最多20个字符

```javascript
function payDemo() {
  let payLoad = JSON.stringify({
      merchantNo: 'm********************5',
      amount: 0.01,
      tradeOrderNo: 'tral' + Date.now() + Math.floor(Math.random() * 100000),
      notifyUrl: 'http://*****/home',
      expireTimeMinute: 200000,
      goodsDesc: '马拉松报名费用'
    })
    alert(payLoad) // 调试使用代码
  if (/iphone/.test(navigator.userAgent.toLowerCase())) {
    let temp;
    if (typeof window.payOrder !== undefined) {
      temp = window.payOrder
    }
    window.payOrder = function (res) {
      console.log(res)
      window.payOrder = temp;
    }
    window.webkit.messageHandlers.payOrder.postMessage(payLoad)
  } else {
    window.ycapp.payOrder(payLoad)
  }
}
```

# 3.基础能力

云城app除了对应给子服务提供设备能力以外，还具有自身的平台能力，按照平台能力的调用规则，即可使用云城对应的平台能力。

## 3.1.卡片消息跳转对应子服务

此能力关联REST接口[服务号向用户推送消息](/pages/development/server/restAPI/childServicePush.md)，按照接口描述调用，用户可以收到对应的卡片消息推送如下：

<img src="/images/childPush.png" width="640"/>

点击卡片消息可以跳转到对应的子服务入口页(参考：[服务创建&发布下的版本发布](/pages/platform/service/README.md#2版本发布))中，如按照推送消息示例中参数进行接口调用，现关注route字段中的value值，由path开始为向子服务中传递的三个参数：path， mobilePhone， name

```json
"route": {
    // iCloudCity://为固定内容，其后跟服务类型如 weex或web，其后跟子服务APPID，即上方appkey的值
    // 其后按序列化字符串拼接所需传递的参数，然后在子服务中进行对应的解析
    "value": "iCloudCity://weex/dae4245d-3/path=index.js&mobilePhone=13530986300&name=zs"
  }
```

可以看到如下图在子服务中可以通过解析url值获取到通过卡片消息传递进来的参数及服务自带的环境变量参数，由此子服务开发者可以通过获取到的不同参数来做对应的处理，如跳转进入次级页面，调用子服务中的其他功能等。

<img src="/images/params.png" width="640" />

